@@ -1,3 +1,4 @@
+from datetime import datetime
 from functools import cache
 from json import dumps
 from json import loads
@@ -86,6 +87,7 @@ class Database(_Database):
 
     @cache
     def clear_cache(self, _m_time: float = None):
+        self.use_bbcode.cache_clear()
         self._load_user_cached.cache_clear()
         self._load_user_stats_cached.cache_clear()
         self._load_submission_cached.cache_clear()
@@ -97,12 +99,16 @@ class Database(_Database):
         self._load_search_cached.cache_clear()
         self._load_files_folder_cached.cache_clear()
         self._load_info_cached.cache_clear()
-        self.logger.info(f"Clearing cache for new m_time {_m_time}")
+        self.logger.info(f"Clearing cache for new m_time {datetime.fromtimestamp(_m_time)}")
 
     @property
     def m_time(self) -> float:
         return self.path.stat().st_mtime
 
+    @cache
+    def use_bbcode(self) -> bool:
+        return self.settings.bbcode
+
     @cache
     def _load_user_cached(self, user: str) -> dict | None:
         return self.users[user]
@@ -189,17 +195,17 @@ class Database(_Database):
         db_table: Table
 
         if (table := table.upper()) == submissions_table.upper():
-            cols_results = [SubmissionsColumns.ID.value, SubmissionsColumns.AUTHOR.value,
-                            SubmissionsColumns.DATE.value, SubmissionsColumns.TITLE.value,
-                            SubmissionsColumns.FILEEXT.value]
+            cols_results = [SubmissionsColumns.ID, SubmissionsColumns.AUTHOR,
+                            SubmissionsColumns.DATE, SubmissionsColumns.TITLE,
+                            SubmissionsColumns.FILEEXT]
             db_table = self.submissions
         elif table == journals_table.upper():
-            cols_results = [JournalsColumns.ID.value, JournalsColumns.AUTHOR.value,
-                            JournalsColumns.DATE.value, JournalsColumns.TITLE.value]
+            cols_results = [JournalsColumns.ID, JournalsColumns.AUTHOR,
+                            JournalsColumns.DATE, JournalsColumns.TITLE]
             db_table = self.journals
         else:
             default_field = "username"
-            cols_results = [UsersColumns.USERNAME.value, UsersColumns.FOLDERS.value, UsersColumns.ACTIVE.value]
+            cols_results = [UsersColumns.USERNAME, UsersColumns.FOLDERS, UsersColumns.ACTIVE]
             db_table = self.users
 
         cols_table: list[str] = [c.name for c in db_table.columns]
