@@ -1,5 +1,4 @@
 from datetime import datetime
-from functools import reduce
 from io import BytesIO
 from json import dumps
 from logging import Logger
@@ -10,6 +9,8 @@ from pathlib import Path
 from re import IGNORECASE
 from re import Pattern
 from re import compile as re_compile
+from re import match
+from re import sub as re_sub
 from secrets import compare_digest
 from sqlite3 import DatabaseError
 from sqlite3 import OperationalError
@@ -21,7 +22,9 @@ from zipfile import ZipFile
 
 from PIL import Image
 from PIL import UnidentifiedImageError
+from bbcode import Parser as BBCodeParser
 from bs4 import BeautifulSoup
+from bs4 import NavigableString
 from bs4.element import Tag
 from chardet import detect as detect_encoding
 from falocalrepo_database.tables import JournalsColumns
@@ -45,6 +48,7 @@ from fastapi.templating import Jinja2Templates
 from htmlmin import minify
 from pydantic import BaseModel
 from pydantic import BaseSettings
+from starlette.middleware.base import BaseHTTPMiddleware
 from uvicorn import run
 from uvicorn.config import LOGGING_CONFIG
 
@@ -110,28 +114,10 @@ settings: Settings = Settings(static_folder=root / "static")
 search_settings: SearchSettings = SearchSettings()
 security: HTTPBasic = HTTPBasic()
 
-bbcode_expressions: list[tuple[Pattern, str]] = [
-    (re_compile(r"\[(/?)([bius]|sup|sub|h\d)]"), r"<\1\2>",),
-    (re_compile(r"\[color=([^]]+)]"), r'<span style="color: \1">,'),
-    (re_compile(r"\[/color]"), "</span>",),
-    (re_compile(r"\[(left|center|right)]"), r'<p style="text-align: \1">',),
-    (re_compile(r"\[/(left|center|right)]"), "</p>",),
-    (re_compile(r"\[quote=([^]]+)]"), r'<blockquote cite="\1">'),
-    (re_compile(r"\[quote]"), "<blockquote>"),
-    (re_compile(r"\[/quote]"), "</blockquote>"),
-    (re_compile(r"\[url=([^]]+)]"), r'<a href="\1">'),
-    (re_compile(r"\[/url]"), "</a>"),
-    (re_compile(r"\[yt]((?:.(?!\[yt]))+)\[/yt]"), r'</a href="\1">\1</a>'),
-    (re_compile(r"(:(?:icon|link)([^:]+):|:([^:]+)icon:)"), r'<a href="/user/\2\3">@\2\3</a>'),
-    (re_compile(r"@(\S+)"), r'<a href="/user/\1">@\1</a>'),
-    (re_compile(r"\n"), "<br/>"),
-    (re_compile(r"[-=]{5,}"), "<hr/>"),
-    (re_compile(r"\[ *(\d*), *(\d*) *, *(\d*) *]"),
-     r'<a href="/submission/\1">PREV</a>|<a href="/submission/\2">FIRST</a>|<a href="/submission/\3">NEXT</a>'),
-    (re_compile(r"\(c\)"), "&copy;"),
-    (re_compile(r"\(r\)"), "&reg;"),
-    (re_compile(r"\(tm\)"), "&trade;"),
-]
+icons: list[str] = ["crying", "derp", "dunno", "embarrassed", "evil", "gift", "huh", "lmao", "love", "nerd", "note",
+                    "oooh", "pleased", "rollingeyes", "sad", "sarcastic", "serious", "sleepy", "smile", "teeth",
+                    "tongue", "veryhappy", "wink", "yelling", "zipped", "angel", "badhairday", "cd", "coffee", "cool",
+                    "whatever"]
 
 app.mount("/static", StaticFiles(directory=settings.static_folder), "static")
 
@@ -147,35 +133,161 @@ def comments_depth(comments: list[dict], depth: int = 0, root_comment: int = Non
             for com in comments]
 
 
-def prepare_comments(comments: list[dict]) -> list[dict]:
-    return [c | {"TEXT": clean_html(c["TEXT"])} for c in flatten_comments(comments_depth(comments, 0))]
+def prepare_comments(comments: list[dict], use_bbcode: bool) -> list[dict]:
+    return [c | {"TEXT": bbcode_to_html(c["TEXT"]) if use_bbcode else clean_html(c["TEXT"])}
+            for c in flatten_comments(comments_depth(comments, 0))]
 
 
-def bbcode_to_html(text: str) -> str:
-    return "".join(reduce(lambda txt, exp: exp[0].sub(exp[1], txt), bbcode_expressions, text.strip()))
+def bbcode_to_html(bbcode: str) -> str:
+    def render_url(_tag_name, value: str, options: dict[str, str], _parent, _context) -> str:
+        return f'<a class="auto_link named_url" href="{options.get("url", "#")}">{value}</a>'
+
+    def render_color(_tag_name, value, options, _parent, _context) -> str:
+        return f'<span class=bbcode style="color:{options.get("color", "inherit")};">{value}</span>'
+
+    def render_quote(_tag_name, value: str, options: dict[str, str], _parent, _context) -> str:
+        author: str = options.get("quote", "")
+        author = f"<span class=bbcode_quote_name>{author} wrote:</span>" if author else ""
+        return f'<span class="bbcode bbcode_quote">{author}{value}</span>'
+
+    def render_tags(tag_name: str, value: str, options: dict[str, str], _parent, _context) -> str:
+        if not options and tag_name.islower():
+            return f"<{tag_name}>{value}</{tag_name}>"
+        return f"[{tag_name} {' '.join(f'{k}={v}' if v else k for k, v in options.items())}]{value}"
+
+    def render_tag(_tag_name, value: str, options: dict[str, str], _parent, _context) -> str:
+        name, *classes = options["tag"].split(".")
+        return f'<{name} class="{" ".join(classes)}">{value}</{name}>'
+
+    def parse_extra(page: BeautifulSoup) -> BeautifulSoup:
+        child: NavigableString
+        child_new: Tag
+        has_match: bool = True
+        while has_match:
+            has_match = False
+            for child in [c for e in page.select("*:not(a)") for c in e.children if isinstance(c, NavigableString)]:
+                if m_ := match(r"(.*)(https?://(?:www\.)?((?:[\w/%#\[\]@*-]|[.,?!'()&~:;=](?! ))+))(.*)", child):
+                    has_match = True
+                    child_new = Tag(name="a", attrs={"class": f"auto_link_shortened", "href": m_[2]})
+                    child_new.insert(0, m_[3].split("?", 1)[0])
+                    child.replaceWith(m_[1], child_new, m_[4])
+                elif m_ := match(rf"(.*):({'|'.join(icons)}):(.*)", child):
+                    has_match = True
+                    child_new = Tag(name="i", attrs={"class": f"smilie {m_[2]}"})
+                    child.replaceWith(m_[1], child_new, m_[3])
+                elif m_ := match(r"(.*)(?:@([a-zA-Z0-9.~_-]+)|:link([a-zA-Z0-9.~_-]+):)(.*)", child):
+                    has_match = True
+                    child_new = Tag(name="a", attrs={"class": "linkusername", "href": f"/user/{m_[2] or m_[3]}"})
+                    child_new.insert(0, m_[2] or m_[3])
+                    child.replaceWith(m_[1], child_new, m_[4])
+                elif m_ := match(r"(.*):(?:icon([a-zA-Z0-9.~_-]+)|([a-zA-Z0-9.~_-]+)icon):(.*)", child):
+                    has_match = True
+                    user: str = m_[2] or m_[3] or ""
+                    child_new = Tag(name="a", attrs={"class": "iconusername", "href": f"/user/{user}"})
+                    child_new_img: Tag = Tag(
+                        name="img",
+                        attrs={"alt": user, "title": user,
+                               "src": f"//a.furaffinity.net/{datetime.now():%Y%m%d}/{clean_username(user)}.gif"})
+                    child_new.insert(0, child_new_img)
+                    if m_[2]:
+                        child_new.insert(1, f"\xA0{m_[2]}")
+                    child.replaceWith(m_[1], child_new, m_[4])
+                elif m_ := match(r"(.*)\[ *(?:(\d+)|-)?, *(?:(\d+)|-)? *, *(?:(\d+)|-)? *](.*)", child):
+                    has_match = True
+                    child_new = Tag(name="span", attrs={"class": "parsed_nav_links"})
+                    child_new_1: Tag | str = "<<<\xA0PREV"
+                    child_new_2: Tag | str = "FIRST"
+                    child_new_3: Tag | str = "NEXT\xA0>>>"
+                    if m_[2]:
+                        child_new_1 = Tag(name="a", attrs={"href": f"/view/{m_[2]}"})
+                        child_new_1.insert(0, "<<<\xA0PREV")
+                    if m_[3]:
+                        child_new_2 = Tag(name="a", attrs={"href": f"/view/{m_[3]}"})
+                        child_new_2.insert(0, "<<<\xA0FIRST")
+                    if m_[4]:
+                        child_new_3 = Tag(name="a", attrs={"href": f"/view/{m_[4]}"})
+                        child_new_3.insert(0, "NEXT\xA0>>>")
+                    child_new.insert(0, child_new_1)
+                    child_new.insert(1, "\xA0|\xA0")
+                    child_new.insert(2, child_new_2)
+                    child_new.insert(3, "\xA0|\xA0")
+                    child_new.insert(4, child_new_3)
+                    child.replaceWith(m_[1], child_new, m_[5])
+
+        for p in page.select("p"):
+            p.replaceWith(*p.children)
+
+        return page
+
+    parser: BBCodeParser = BBCodeParser(install_defaults=False, replace_links=False, replace_cosmetic=True)
+    parser.REPLACE_ESCAPE = (
+        ("&", "&amp;"),
+        ("<", "&lt;"),
+        (">", "&gt;"),
+    )
+    parser.REPLACE_COSMETIC = (
+        ("(c)", "&copy;"),
+        ("(r)", "&reg;"),
+        ("(tm)", "&trade;"),
+    )
+
+    for tag in ("i", "b", "u", "s", "sub", "sup", "h1", "h2", "h3", "h3", "h4", "h5", "h6"):
+        parser.add_formatter(tag, render_tags)
+    for align in ("left", "center", "right"):
+        parser.add_simple_formatter(align, f'<code class="bbcode bbcode_{align}">%(value)s</code>')
+
+    parser.add_simple_formatter("spoiler", '<span class="bbcode bbcode_spoiler">%(value)s</span>')
+    parser.add_simple_formatter("url", '<a class="auto_link named_link">%(value)s</a>')
+    parser.add_simple_formatter(
+        "iconusername",
+        f'<a class=iconusername href="/user/%(value)s">'
+        f'<img alt="%(value)s" title="%(value)s" src="//a.furaffinity.net/{datetime.now():%Y%m%d}/%(value)s.gif">'
+        f'%(value)s'
+        f'</a>'
+    )
+    parser.add_simple_formatter(
+        "usernameicon",
+        f'<a class=iconusername href="/user/%(value)s">'
+        f'<img alt="%(value)s" title="%(value)s" src="//a.furaffinity.net/{datetime.now():%Y%m%d}/%(value)s.gif">'
+        f'</a>'
+    )
+    parser.add_simple_formatter("linkusername", '<a class=linkusername href="/user/%(value)s">%(value)s</a>')
+    parser.add_simple_formatter("hr", "<hr>", standalone=True)
+
+    parser.add_formatter("url", render_url)
+    parser.add_formatter("color", render_color)
+    parser.add_formatter("quote", render_quote)
+    parser.add_formatter("tag", render_tag)
+
+    bbcode = re_sub(r"-{5,}", "[hr]", bbcode)
+
+    result_page: BeautifulSoup = parse_extra(BeautifulSoup(parser.format(bbcode), "lxml"))
+    return (result_page.select_one("html > body") or result_page).decode_contents()
 
 
 def clean_html(html: str) -> str:
     html_parsed: BeautifulSoup = BeautifulSoup(html, "lxml")
     for icon in html_parsed.select("a.iconusername > img"):
-        parent: Tag = icon.parent
-        parent.clear()
-        parent.insert(0, f"@{icon.attrs['title']}")
-    for code in html_parsed.select("code"):
-        code.name = "div"
+        icon.attrs["hidden"] = "true"
+        icon.attrs["onload"] = "this.hidden = false"
     for link in html_parsed.select("a[href*='furaffinity.net']"):
         link["href"] = "/" + fa_link.sub("", link.attrs["href"]).strip("/")
     return str(html_parsed)
 
 
-def serialise_entry(entry: Any) -> Any:
+def prepare_html(html: str, use_bbcode: bool) -> str:
+    return clean_html(bbcode_to_html(html)) if use_bbcode else clean_html(html)
+
+
+def serialise_entry(entry: Any, convert_datetime: bool = False, lowercase_keys: bool = True) -> Any:
     if isinstance(entry, dict):
-        return {k: serialise_entry(v) for k, v in entry.items()}
+        return {(k.lower() if lowercase_keys else k): serialise_entry(v, convert_datetime, lowercase_keys)
+                for k, v in entry.items()}
     elif isinstance(entry, (list, tuple)):
-        return list(map(serialise_entry, entry))
+        return [serialise_entry(e, convert_datetime, lowercase_keys) for e in entry]
     elif isinstance(entry, set):
-        return sorted(map(serialise_entry, entry))
-    elif isinstance(entry, datetime):
+        return sorted((serialise_entry(e, convert_datetime, lowercase_keys) for e in entry))
+    elif isinstance(entry, datetime) and convert_datetime:
         return str(entry)
     else:
         return entry
@@ -215,7 +327,8 @@ async def auth_middleware(request: Request, call_next: Callable[[Request], Corou
 @app.on_event("startup")
 def log_settings():
     logger.info(f"Version: {__version__}")
-    logger.info(f"Using database: {settings.database.path} ({settings.database.version})")
+    logger.info(f"Using database: {settings.database.path} ({settings.database.version})" +
+                (" (BBCode)" if settings.database.use_bbcode() else ""))
     logger.info(f"Using SSL certificate: {settings.ssl_cert}") if settings.ssl_cert else None
     logger.info(f"Using SSL private key: {settings.ssl_key}") if settings.ssl_key else None
     logger.info(f"Using HTTP Basic authentication") if settings.username or settings.password else None
@@ -303,28 +416,28 @@ async def redirect_submission(id_: int):
 @app.get("/search/gallery/{username}/", response_class=HTMLResponse)
 async def serve_user_gallery(request: Request, username: str):
     return await serve_search(request, "submissions", f"Gallery {username}",
-                              {"query": f'@author "{clean_username(username)}" @folder "gallery"'})
+                              {"query": f'@author ^{clean_username(username)}$ @folder "gallery"'})
 
 
 @app.get("/scraps/{username}/", response_class=HTMLResponse)
 @app.get("/search/scraps/{username}/", response_class=HTMLResponse)
 async def serve_user_scraps(request: Request, username: str):
     return await serve_search(request, "submissions", f"Scraps {username}",
-                              {"query": f'@author "{clean_username(username)}" @folder "scraps"'})
+                              {"query": f'@author ^{clean_username(username)}$ @folder "scraps"'})
 
 
 @app.get("/submissions/{username}/", response_class=HTMLResponse)
 @app.get("/search/submissions/{username}/", response_class=HTMLResponse)
 async def serve_user_submissions(request: Request, username: str):
     return await serve_search(request, "submissions", f"Submissions {username}",
-                              {"query": f'@author "{clean_username(username)}"'})
+                              {"query": f'@author ^{clean_username(username)}$'})
 
 
 @app.get("/journals/{username}/", response_class=HTMLResponse)
 @app.get("/search/journals/{username}/", response_class=HTMLResponse)
 async def serve_user_journals(request: Request, username: str):
     return await serve_search(request, "journals", f"Journals {username}",
-                              {"query": f'@author "{clean_username(username)}"'})
+                              {"query": f'@author ^{clean_username(username)}$'})
 
 
 @app.get("/favorites/{username}/", response_class=HTMLResponse)
@@ -371,9 +484,9 @@ async def serve_settings(request: Request):
         "app": app.title,
         "title": "Search Settings",
         "tables": [users_table, submissions_table, journals_table],
-        "columns": {users_table: [c.name for c in UsersColumns],
-                    submissions_table: [c.name for c in SubmissionsColumns] + ["relevance"],
-                    journals_table: [c.name for c in JournalsColumns]},
+        "columns": {users_table: [c.name for c in UsersColumns.as_list()],
+                    submissions_table: [c.name for c in SubmissionsColumns.as_list()] + ["relevance"],
+                    journals_table: [c.name for c in JournalsColumns.as_list()]},
         "settings": search_settings,
         "default_sort": default_sort,
         "default_order": default_order,
@@ -401,7 +514,7 @@ async def serve_user(request: Request, username: str):
     if username != (username_clean := clean_username(username)):
         return RedirectResponse(app.url_path_for(serve_user.__name__, username=username_clean))
 
-    user_entry: dict | None = settings.database.load_user(username)
+    user_entry: dict = settings.database.load_user(username) or {}
     user_stats: dict[str, int] = settings.database.load_user_stats(username)
     p, n = settings.database.load_prev_next(users_table, username) if user_entry else (0, 0)
 
@@ -409,21 +522,32 @@ async def serve_user(request: Request, username: str):
         "app": app.title,
         "title": username,
         "user": username,
-        "folders": user_entry["FOLDERS"] if user_entry else [],
-        "active": user_entry["ACTIVE"] if user_entry else True,
+        "folders": user_entry.get("FOLDERS", []),
+        "active": user_entry.get("ACTIVE", True),
         "gallery_length": user_stats["gallery"],
         "scraps_length": user_stats["scraps"],
         "favorites_length": user_stats["favorites"],
         "mentions_length": user_stats["mentions"],
         "journals_length": user_stats["journals"],
-        "userpage": clean_html(user_entry["USERPAGE"]) if user_entry else "",
-        "in_database": user_entry is not None,
+        "userpage": prepare_html(user_entry.get("USERPAGE", ""), settings.database.use_bbcode()),
+        "userpage_bbcode": user_entry.get("USERPAGE", None) if settings.database.use_bbcode() else None,
+        "icon": f"//a.furaffinity.net/{datetime.now():%Y%m%d}/{clean_username(username)}.gif",
+        "in_database": bool(user_entry),
         "prev": p,
         "next": n,
         "request": request}),
         remove_comments=True))
 
 
+@app.get("/user/{username}/icon/")
+@app.get("/user/{username}/thumbnail/")
+async def serve_user_thumbnail(request: Request, username: str):
+    if username != (username_clean := clean_username(username)):
+        return RedirectResponse(app.url_path_for(serve_user_thumbnail.__name__, username=username_clean))
+
+    return RedirectResponse(f"https://a.furaffinity.net/{datetime.now():%Y%m%d}/{username}.gif")
+
+
 @app.get("/search/{table}/", response_class=HTMLResponse)
 async def serve_search(request: Request, table: str, title: str = None, args: dict[str, str] = None):
     if (table := table.upper()) not in (submissions_table, journals_table, users_table):
@@ -465,7 +589,7 @@ async def serve_search(request: Request, table: str, title: str = None, args: di
         "sort": sort,
         "order": order,
         "view": view,
-        "thumbnails": table == submissions_table,
+        "thumbnails": table in (submissions_table, users_table),
         "columns_table": columns_table,
         "columns_results": columns_results,
         "columns_list": columns_list,
@@ -492,14 +616,19 @@ async def serve_submission(request: Request, id_: int):
     return HTMLResponse(minify(templates.get_template("submission.html").render({
         "app": app.title,
         "title": f"{sub['TITLE']} by {sub['AUTHOR']}",
-        "submission": sub | {"DESCRIPTION": clean_html(sub["DESCRIPTION"])},
+        "submission": sub | {
+            "DESCRIPTION": prepare_html(sub["DESCRIPTION"], settings.database.use_bbcode()),
+            "FOOTER": prepare_html(sub["FOOTER"], settings.database.use_bbcode()),
+            "DESCRIPTION_BBCODE": sub["DESCRIPTION"].strip() or None if settings.database.use_bbcode() else None,
+            "FOOTER_BBCODE": sub["FOOTER"].strip() or None if settings.database.use_bbcode() else None,
+        },
         "files_text": [
             bbcode_to_html(fs[i].read_text(detect_encoding(fs[i].read_bytes())["encoding"], "ignore"))
             if ext.lower() == "txt" else ""
-            for i, ext in enumerate(sub['FILEEXT'])] if fs else [],
+            for i, ext in enumerate(sub['FILEEXT']) if fs[i].is_file()] if fs else [],
         "filenames": [f"submission{('.' + ext) * bool(ext)}" for ext in sub['FILEEXT']],
         "filenames_id": [f"{sub['ID']:010d}{('.' + ext) * bool(ext)}" for ext in sub['FILEEXT']],
-        "comments": prepare_comments(settings.database.load_submission_comments(id_)),
+        "comments": prepare_comments(settings.database.load_submission_comments(id_), settings.database.use_bbcode()),
         "prev": p,
         "next": n,
         "request": request}),
@@ -576,10 +705,11 @@ async def serve_submission_zip(id_: int, _filename=None):
         for sub_file in sub_files:
             z.writestr(sub_file.name, sub_file.read_bytes())
         z.writestr(sub_thumb.name, sub_thumb.read_bytes()) if sub_thumb else None
-        z.writestr("description.html", sub["DESCRIPTION"].encode())
-        z.writestr("metadata.json", dumps({k: v for k, v in serialise_entry(sub).items()
-                                           if k != "DESCRIPTION"}).encode())
-        z.writestr("comments.json", dumps([serialise_entry(c)
+        z.writestr("description.txt" if settings.database.use_bbcode() else "description.html",
+                   sub["DESCRIPTION"].encode())
+        z.writestr("metadata.json", dumps({k: v for k, v in serialise_entry(sub, convert_datetime=True).items()
+                                           if k != "description"}).encode())
+        z.writestr("comments.json", dumps([serialise_entry(c, convert_datetime=True)
                                            for c in settings.database.load_submission_comments(id_)]).encode())
 
     f_obj.seek(0)
@@ -596,8 +726,15 @@ async def serve_journal(request: Request, id_: int):
     return HTMLResponse(minify(templates.get_template("journal.html").render({
         "app": app.title,
         "title": f"{jrnl['TITLE']} by {jrnl['AUTHOR']}",
-        "journal": jrnl | {"CONTENT": clean_html(jrnl["CONTENT"])},
-        "comments": prepare_comments(settings.database.load_journal_comments(id_)),
+        "journal": jrnl | {
+            "CONTENT": prepare_html(jrnl["CONTENT"], settings.database.use_bbcode()),
+            "HEADER": prepare_html(jrnl["HEADER"], settings.database.use_bbcode()),
+            "FOOTER": prepare_html(jrnl["FOOTER"], settings.database.use_bbcode()),
+            "CONTENT_BBCODE": jrnl["CONTENT"].strip() or None if settings.database.use_bbcode() else None,
+            "HEADER_BBCODE": jrnl["HEADER"].strip() or None if settings.database.use_bbcode() else None,
+            "FOOTER_BBCODE": jrnl["FOOTER"].strip() or None if settings.database.use_bbcode() else None,
+        },
+        "comments": prepare_comments(settings.database.load_journal_comments(id_), settings.database.use_bbcode()),
         "prev": p,
         "next": n,
         "request": request}),
@@ -611,10 +748,10 @@ async def serve_journal_zip(id_: int, _filename=None):
         raise HTTPException(404)
 
     with ZipFile(f_obj := BytesIO(), "w") as z:
-        z.writestr("content.html", jrnl["CONTENT"].encode())
-        z.writestr("metadata.json", dumps({k: v for k, v in serialise_entry(jrnl).items()
-                                           if k != "CONTENT"}).encode())
-        z.writestr("comments.json", dumps([serialise_entry(c)
+        z.writestr("content.txt" if settings.database.use_bbcode() else "content.html", jrnl["CONTENT"].encode())
+        z.writestr("metadata.json", dumps({k: v for k, v in serialise_entry(jrnl, convert_datetime=True).items()
+                                           if k != "content"}).encode())
+        z.writestr("comments.json", dumps([serialise_entry(c, convert_datetime=True)
                                            for c in settings.database.load_journal_comments(id_)]).encode())
 
     f_obj.seek(0)
@@ -649,7 +786,7 @@ async def serve_search_json(request: Request, table: str, query_data: SearchQuer
             "column_id": col_id,
             "limit": query_data.limit,
             "offset": query_data.offset,
-            "results": results[query_data.offset:query_data.offset + query_data.limit],
+            "results": serialise_entry(results[query_data.offset:query_data.offset + query_data.limit]),
             "results_total": len(results)}
 
 
@@ -659,7 +796,7 @@ async def serve_submission_json(id_: int):
     if not (s := settings.database.load_submission_uncached(id_)):
         raise HTTPException(404)
     else:
-        return s | {"COMMENTS": settings.database.load_submission_comments_uncached(id_)}
+        return serialise_entry(s | {"comments": settings.database.load_submission_comments_uncached(id_)})
 
 
 @app.get("/json/journal/{id_}/", response_class=JSONResponse)
@@ -668,17 +805,17 @@ async def serve_journal_json(id_: int):
     if not (j := settings.database.load_journal_uncached(id_)):
         raise HTTPException(404)
     else:
-        return j | {"COMMENTS": settings.database.load_journal_comments_uncached(id_)}
+        return serialise_entry(j | {"comments": settings.database.load_journal_comments_uncached(id_)})
 
 
 @app.get("/json/user/{username}/", response_class=JSONResponse)
 @app.post("/json/user/{username}/", response_class=JSONResponse)
 async def serve_user_json(username: str):
     username = clean_username(username)
-    user_entry: dict | None = settings.database.load_user_uncached(username)
+    user_entry: dict = settings.database.load_user_uncached(username) or {}
     user_stats: dict[str, int] = settings.database.load_user_stats_uncached(username)
 
-    return {"username": username, **{k.lower(): v for k, v in (user_entry or {}).items()}, "length": user_stats}
+    return serialise_entry({"username": username, "length": user_stats} | user_entry)
 
 
 def run_redirect(host: str, port_listen: int, port_redirect: int):
@@ -698,24 +835,30 @@ def server(database_path: str | PathLike, host: str = "0.0.0.0", port: int = Non
            browser: bool = True):
     if redirect_port:
         return run_redirect(host, port, redirect_port)
+
     settings.precache = precache
     settings.open_browser = browser
+
     if authentication:
         settings.username = authentication.split(":")[0]
         settings.password = authentication.split(":", 1)[1] if ":" in authentication else ""
-        app.middleware("http")(auth_middleware)
-    run_args: dict[str, Any] = {}
+        app.add_middleware(BaseHTTPMiddleware, dispatch=auth_middleware)
+
     if ssl_cert and ssl_key:
         settings.ssl_cert, settings.ssl_key = Path(ssl_cert), Path(ssl_key)
         if not settings.ssl_cert.is_file():
             raise FileNotFoundError(f"SSL certificate {settings.ssl_cert}")
         elif not settings.ssl_key.is_file():
             raise FileNotFoundError(f"SSL private key {settings.ssl_key}")
-        run_args |= {"port": port or 443, "ssl_certfile": settings.ssl_cert, "ssl_keyfile": settings.ssl_key}
+        port = port or 443
+
     settings.address = f"{'https' if settings.ssl_cert else 'http'}://" \
                        f"{'localhost' if host == '0.0.0.0' else host}" \
                        f"{f':{port}' if port else ''}"
-    run_args |= {"port": run_args.get("port", port) or 80}
+
     with Database(Path(database_path).resolve(), logger) as settings.database:
         search_settings.load(settings.database.load_settings("SEARCH"))
-        run(app, host=host, **run_args, log_config=LOGGING_CONFIG)
+        run(app, host=host, port=port,
+            ssl_certfile=settings.ssl_cert,
+            ssl_keyfile=str(settings.ssl_key) if settings.ssl_key else None,
+            log_config=LOGGING_CONFIG)
